package com.foundeo.spellchecker;

/* ******************************************************************************
 * Author           Peter Freitag 
 * description      Output Abstraction layer for Spell Checker
 ******************************************************************************/



/*******************************************************************************
 * Open Source Class to contain spelling output formats, This class is "abstract"
 * So you do not create an instance of this class, all methods of this class 
 * should be static.  Each method Serializes mispelled words and their suggestions
 * into String's.
 * 
 * @author Peter Freitag
 * @see MisspelledCollection
 ******************************************************************************/
public abstract class SpellingOutput 
{
    /** The System Line Seperator, the new line char sequence */
    public static final String BR = System.getProperty("line.separator");
    
    /***************************************************************************
     * This function allows you to tie names to the formatting functions.
     * using this attributes can be easily mapped.  Eg. format="wddx" maps to
     * getWDDXOutput().  For example in CFX_JSpellCheck calls this function based
     * using the value inputted as format="javascript"
     *
     * @param format A string mapping to the output function
     * @param mc A MisspelledCollection reference of mispelled words to be passed
     *      into the output function.
     * @return the result of the mapped output.
     **************************************************************************/
    public static final String getOutputByName(String format, MisspelledCollection mc) 
    {
        if (format.toLowerCase().equals("javascript")) 
        {
            return getJSOutput(mc);
        }
        else if (format.toLowerCase().equals("wddx"))
        {
            return getWDDXOutput(mc);
        }
        //add custom output formats here
            
        
        else 
        {
            return "No function matches the format: " + format + " you specified";
        }
    }
    
    /***************************************************************************
     * Java Script output function formats two Java Script arrays one with the 
     * mispelled  word list, and the other a two dimensional array that 
     * maps the suggestions to a mispelled word.  It also sets a variable called
     * numMispelled which contians the total number of mispelled words.
     *
     * @param mispelled the return of checkSpelling()
     * @return a formatted output of the mispelled words and their suggestions
     **************************************************************************/
    public static final String getJSOutput(MisspelledCollection mispelled)
    {   
        int size = mispelled.size();
        //*Performance - StringBuffer's are faster than String concatination (+)
        StringBuffer output = new StringBuffer(size*500);     //output buffer
        output.append("var numMispelled = ");
            output.append(size);
            output.append(";"); 
            output.append(BR);
        
        if (size > 0) 
        {
            output.append("var mispelled = new Array(");
                output.append(size);
                output.append(");");
                output.append(BR);
            output.append("var suggestions = new Array(");
                output.append(size);
                output.append(");");
                output.append(BR);
                       
            //for each mispelled word
            for (int i=0;i<size;i++) 
            {
                String word = mispelled.getWordAt(i);
                output.append("mispelled[");
                    output.append(i);
                    output.append("] = \"");
                    output.append(word);
                    output.append("\";"); 
                    output.append(BR);
                    
                String[] suggestions = (String[])mispelled.getSuggestionsAt(i);
                
                output.append("suggestions[");
                    output.append(i);
                    output.append("] = new Array("); 
                    output.append(suggestions.length);
                    output.append(");");
                    output.append(BR);
                
                //for each suggestion
                for (int n=0;n<suggestions.length;n++) 
                {
                    output.append("suggestions[");
                        output.append(i);
                        output.append("][");
                        output.append(n);
                        output.append("] = \"");
                        output.append(suggestions[n]);
                        output.append("\";");
                        output.append(BR);
                }
            } 
        }
        
        return output.toString();
    }
 
    /***************************************************************************
     * WDDX output function formats two arrays one with the 
     * mispelled  word list, and the other a two dimensional array that 
     * maps the suggestions to a mispelled word.  It also sets a variable called
     * numMispelled which contians the total number of mispelled words.
     * See <a href="http://www.wddx.org" target="_blank">http://www.wddx.org</a>
     *
     * @param mispelled the return of checkSpelling()
     * @return a WDDX packet mapping mispelled words and their suggestions
     **************************************************************************/
    public static final String getWDDXOutput(MisspelledCollection mispelled) 
    {
        int size = mispelled.size();
        //*Performance - StringBuffer's are faster than String concatination (+)
        StringBuffer output = new StringBuffer(size*500);
        output.append("<?xml version='1.0'?>"); 
            output.append(BR); 
        output.append("<!DOCTYPE wddxPacket SYSTEM 'wddx_0100.dtd'>");
            output.append(BR);
        output.append("<wddxPacket version='1.0'>");
            output.append(BR);
        output.append("\t<header/>");
            output.append(BR);
        output.append("\t<data>");
            output.append(BR);
        output.append("\t\t<struct>");
            output.append(BR);
        output.append("\t\t\t<var name='numMispelled'>");
            output.append(BR);
        output.append("\t\t\t\t<number>");
            output.append(size);
            output.append("</number>");
            output.append(BR);
        output.append("\t\t\t</var>");
            output.append(BR);
        
        if (size > 0) 
        {
            output.append("\t\t\t<var name='mispelled'>"); 
                output.append(BR);
            output.append("\t\t\t\t<array length='" + size + "'>"); 
                output.append(BR);
                        
            //for each mispelled word
            for (int i=0;i<size;i++) 
            {
                String word = mispelled.getWordAt(i);
                output.append("\t\t\t\t\t<string>"); 
                    output.append(word); 
                    output.append("</string>"); 
                    output.append(BR);
            }
            output.append("\t\t\t\t</array>"); 
                output.append(BR);
            output.append("\t\t\t</var>"); 
                output.append(BR);
            output.append("\t\t\t<var name='suggestions'>"); 
                output.append(BR);
            output.append("\t\t\t\t<array length='"); 
                output.append(size);
                output.append("'>");
                output.append(BR);
            
            //for each mispelled word
            for (int i=0;i<size;i++) 
            {
                String[] suggestions = mispelled.getSuggestionsAt(i);
                output.append("\t\t\t\t\t<array length='");
                    output.append(suggestions.length + "'>" );
                    output.append(BR);
                
                //for each suggestion
                for (int n=0;n<suggestions.length;n++) 
                {
                    output.append("\t\t\t\t\t\t<string>");
                        output.append(suggestions[n]);
                        output.append("</string>"); 
                        output.append(BR);
                }
                output.append("\t\t\t\t\t</array>"); 
                    output.append(BR);
            }
            
            output.append("\t\t\t\t</array>"); 
                output.append(BR);
            output.append("\t\t\t</var>"); 
                output.append(BR);
        }      
        output.append("\t\t</struct>"); 
            output.append(BR);
        output.append("\t</data>"); 
            output.append(BR);
        output.append("</wddxPacket>"); 
            output.append(BR);
        
        return output.toString();
    }
}
    